#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);

my $envelope         = $ARGV[0];
my $file             = $envelope.".txt";
my $number_phislices = $ARGV[1];
my $number_rhoslices = $ARGV[2];
my $shield_material  = $ARGV[3];

my $rmin = 1;
my $rmax = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

use Getopt::Long;
use Math::Trig;

# dimensions in cm

my $r0 = 28.8;   # between CTOF
my $r1 = 38.5;   # and Solenoid

my $length = 66.0/2.0;     # same as CTOF paddles

my $paddles_gap = 0.01;    # gap betweens paddles (both directions)
my $mother_gap  = 0.12;    # mother volume gap

my $cnd_shield_thickness = 0.2;
my $cnd_shield_r0        = $r0 - $mother_gap;

my $wrap_thickness  = 0.003; # thickness of wrapping round paddles

my $dR = ($r1-$r0-$paddles_gap*($number_rhoslices-1) - 2*$mother_gap -$cnd_shield_thickness) / $number_rhoslices;  # height of one region

my $angle_slice = 360.0/$number_phislices;

my @pcolor = ('33dd66', '33bb88', '33aaaa', '3399bb');


# Mother Volume
sub make_CND
{
 my $IR = $r0		- $mother_gap;
 my $OR = $r1		+ $mother_gap;
 my $LL = $length + $mother_gap;

 $detector{"name"}        = $envelope;
 $detector{"mother"}      = "root";
 $detector{"description"} = "Central Neutron Detector";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "33bb99";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$IR*cm $OR*cm $LL*cm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}


# Paddles
sub make_CND_paddles
{
  for(my $j=1; $j<=$number_rhoslices; $j++)
  {
	my $innerRadius = $r0 + ($j-1) *$dR + ($j-1)*$paddles_gap + $mother_gap;
	my $outerRadius = $innerRadius + $dR;
	my $R = ($innerRadius + $outerRadius) / 2.0;

	my $side1 = 2.0*$innerRadius*sin(rad($angle_slice/2.0));
	my $side2 = 2.0*$outerRadius*sin(rad($angle_slice/2.0));

	for(my $i=1; $i<=$number_phislices; $i++)
	{
		my $pnumber = cnumber($i-1, 10);
		my $theta   = ($i-1)*$angle_slice;
		my $x       = sprintf("%.3f", $R*cos(rad($theta)));
		my $y       = sprintf("%.3f", $R*sin(rad($theta)));
		my $z       = "0.";
		my $theta2  = $theta + 90;
		my $dx1     = ($side2 - $paddles_gap)/2.0;
		my $dx2     = ($side1 - $paddles_gap)/2.0;
		my $dy      = $length;
		my $dz      = ($outerRadius - $innerRadius)/2.0;

		$detector{"name"}        =  "CND_Layer$j"."_Paddle_$pnumber";
		$detector{"mother"}      = $envelope;
		$detector{"description"} = "Central Neutron Detector, Layer $j, Scintillator $i";
		$detector{"pos"}         = "$x*cm $y*cm $z*cm";
		$detector{"rotation"}    = "90*deg $theta2*deg 0*deg";
		$detector{"color"}       = $pcolor[$j-1];
		$detector{"type"}        =  "Trd";
		$detector{"dimensions"}  = "$dx1*cm $dx2*cm $dy*cm $dy*cm $dz*cm";
		$detector{"material"}    = "ScintillatorB";
		$detector{"mfield"}      =  "no";
		$detector{"ncopy"}       = $pnumber;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "CND";
		$detector{"hit_type"}    = "CND";
		$detector{"identifiers"} = "layer manual $j paddle manual $i";

		print_det(\%detector, $file);
    }
  }
}

# Paddle wrapping
sub make_paddle_wrapping
{
  for(my $j=1; $j<=2*$number_rhoslices; $j++)
  {
	my $innerRadius = $r0 + $mother_gap +  (2*$j - 1 + (-1)**$j)*$dR/4  +  (2*$j - 3 - (-1)**$j)*$paddles_gap/4 - (1 - (-1)**$j)*$wrap_thickness/2;
	my $outerRadius = $innerRadius + $wrap_thickness;
	my $R = ($innerRadius + $outerRadius) / 2.0;

	my $side1 = 2.0*$innerRadius*sin(rad($angle_slice/2.0));
	my $side2 = 2.0*$outerRadius*sin(rad($angle_slice/2.0));

	for(my $i=1; $i<=$number_phislices; $i++)
	{
		my $pnumber = cnumber($i-1, 10);
		my $theta   = ($i-1)*$angle_slice;
		my $x       = sprintf("%.3f", $R*cos(rad($theta)));
		my $y       = sprintf("%.3f", $R*sin(rad($theta)));
		my $z       = "0.";
		my $theta2  = $theta + 90;
		my $dx1     = ($side2 - $paddles_gap)/2.0;
		my $dx2     = ($side1 - $paddles_gap)/2.0;
		my $dy      = $length;
		my $dz      = ($outerRadius - $innerRadius)/2.0;

		$detector{"name"}        =  "CND_paddle_wrap_Layer$j"."_Paddle_$pnumber";
		$detector{"mother"}      = $envelope;
		$detector{"description"} = "Central Neutron Detector, Paddle Wrapping Layer $j, Scintillator $i";
		$detector{"pos"}         = "$x*cm $y*cm $z*cm";
		$detector{"rotation"}    = "90*deg $theta2*deg 0*deg";
		$detector{"color"}       = "88aaff";
		$detector{"type"}        =  "Trd";
		$detector{"dimensions"}  = "$dx1*cm $dx2*cm $dy*cm $dy*cm $dz*cm";
		$detector{"material"}    = "G4_Al";
		$detector{"mfield"}      =  "no";
		$detector{"ncopy"}       = $pnumber;
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 1;
		$detector{"style"}       = 1;
		$detector{"sensitivity"} = "CND";
		$detector{"hit_type"}    = "CND";
		$detector{"identifiers"} = "layer manual $j paddle manual $i";

		print_det(\%detector, $file);
    }
  }
}



# CND Shield Volume
sub make_CND_shield
{
	my $IR = $cnd_shield_r0;
	my $OR = $IR + $cnd_shield_thickness;
	my $LL = $length;
	
	$detector{"name"}        = "cnd_shield";
	$detector{"mother"}      = $envelope;
	$detector{"description"} = "Central Neutron Detector Shield";
	$detector{"pos"}         = "0*cm 0*cm 0*cm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = "992222";
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$IR*cm $OR*cm $LL*cm 0*deg 360*deg";
	$detector{"material"}    = "$shield_material";
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";
	
	print_det(\%detector, $file);
}

make_CND();
make_CND_paddles();
make_paddle_wrapping();
make_CND_shield();
