#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;

# All dimensions in mm

my $envelope = 'BST';
my $file     = 'BST.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;


# General description
my @radius             = (65.117   , 92.681    , 119.961   , 160.660 );   # Inner radii of barrel SVT
my @number_of_modules  = (3        , 3         , 3         , 3       );   # Number of modules for each layer
my @nsegments          = (10       , 14        , 18        , 24      );   # Number of segments in each layer
my @starting_point     = (-219.856 , -180.490  , -141.53   , -83.406 );   # Z starting_point of the moduli (pcboard is behind this)
my @starting_theta     = (-90      , -90       , -80       , -90     );   # Starting angle of the first segment. 360/18/2 layer 3
my @dtheta             = (0        , 0         , 0         , 0       );   # Delta theta
my @bst_copper_length  = (105.99   , 145.36    , 184.32    , 242.44  );   # From the Pitch Adapter to the Cold Plate
 
 
my $nlayers = 4;

my $bst_ir = 50;         # Mother Volume IR
my $bst_or = 174;        # Mother Volume OR
my $bst_dz = 831.0/2.0;  # Mother Volume half length

my @SL_ir = (60, 85,  112, 154);    # Individual superlayers IR, OR
my @SL_or = (75, 103, 130, 171);
my $SL_dz = 800.0/2.0;

for(my $i = 0; $i < $nlayers; $i++)
{
 $dtheta[$i] = 360/$nsegments[$i];
}

#######################
# Dimensions:
# Each segment is a Box
#######################
my $rohacell_length       = 353.07;          # Rohacell support length 3xsensor + 3xgap + 2mm end + 13.5 plastic + 2mm pitch adapter = 352.705
my $rohacell_height       = 38.00;           # Rohacell support transverse dimension
my $plastic_length        = 13.50;           # Plastic Tab length
my $plastic_height        = 20.00;           # Palstic Tab transverse dimension
my $roh_carbon_gap        = 4.0;             # Z Gap between rohacell and carbon fiber (Pitch Adapter)
my $module_length         = 111.625;         # Module length
my $module_height         = 42.195;          # Module transverse dimension
my $carbon_fiber_length   = 401.900;         # Carbon Fiber support length
my $readout_plate_length  = 41.00;           # Readout plate length
my $pcboard_length        = 37.00;           # HFCB length
my $pcboard_height        = 36.75;           # HFCB transverse dimension
my $wirebond_length       = 3.00;            # Wirebond length
my $chip_length           = 5.0;             # chip length
my $chip_height           = 8.0;             # chip transverse dimension
my $chip_displacement     = 8.0;             # chip displacement from center
my $pitch_adapter_length  = 4;               # Pitch Adapter Length
my $bst_support_height    = 42;
my $silicon_gap_width     = 0.11;



# widths
my $silicon_width      = 0.320;    # Silicon sensor width
my $bus_cable_width    = 0.031;    # Bus Cable width
my $epoxy_width        = 0.100;    # Epoxy glue width
my $carbon_fiber_width = 0.250;    # Carbon fiber width
my $rohacell_width     = 2.500;    # Rohacell width
my $plastic_width      = 2.490;    # Rohacell width
my $pcboard_width      = 0.500;    # PC Board width
my $chip_width         = 0.500;    # chip width
my $wirebond_width     = 0.300;    # Wirebond width
my $bst_support_width  = 2.500;    # Support Copper width


# Starts from the bottom silicon up to the top silicon

my @module_composition = ($silicon_width,
													$epoxy_width,
													$bus_cable_width,
													$carbon_fiber_width,
													$rohacell_width,
													$carbon_fiber_width,
													$bus_cable_width,
													$epoxy_width,
													$silicon_width);


# G4 materials
my $silicon_material      = 'G4_Si';
my $rohacell_material     = 'Rohacell';
my $carbon_fiber_material = 'CarbonFiber';
my $epoxy_material        = 'Epoxy';
my $bus_cable_material    = 'BusCable';
my $pcboard_material      = 'PCBoardM';
my $paboard_material      = 'G4_SILICON_DIOXIDE';
my $chip_material         = 'PCBoardM';
my $wirebond_material     = 'svtwirebond';
my $bst_support_material  = 'G4_Cu';


# G4 colors
my @silicon_color      = ('353540', '404045', '353540');
my $bus_cable_color    = '1205a1';
my $epoxy_color        = 'e200e1';
my $rohacell_color     = 'e10000';
my $carbon_fiber_color = '252020';
my $pcboard_color      = '0000ff';
my $paboard_color      = 'ff0000';
my $chip_color         = 'fff600';
my $wirebond_color     = 'a5af9a';
my $bst_support_color  = 'aa0066';


my $microgap_width = 0.002;

sub r_placement
{
	my $l  = shift;
	my $ml = shift;

	my $r = $radius[$l];
	
	for(my $z = 0; $z < $ml-1; $z++)
	{
		$r = $r + $module_composition[$z];
		if($ml>1) 
		{
			$r = $r + $microgap_width;
		}
	}
	$r = $r + $module_composition[$ml-1]/2.0;

	return $r;
}

sub pos_t
{
 my $R = shift;
 my $l = shift;
 my $s = shift;

 my $theta     = 90 + $starting_theta[$l] + $s*$dtheta[$l] ;
 my $x         = sprintf("%.3f",$R*cos(rad($theta)));
 my $y         = sprintf("%.3f",$R*sin(rad($theta)));
 return "$x*mm $y*mm";
}

sub rot
{
 my $R = shift;
 my $l = shift;
 my $s = shift;

 my $theta_rot = $starting_theta[$l] - $s*$dtheta[$l];
 if($l == 2)  {$theta_rot -= 20;}
 if($s == 199) {$theta_rot -= 7.5;}

 return "0*deg 0*deg $theta_rot*deg";
}

sub pos_t_chip
{
 my $R = shift;
 my $l = shift;
 my $t = shift;
 my $s = shift;

 my $dth = asind($chip_displacement / $R);

 my $theta = 0;

 if($t == 1){ $theta     = 90 - $dth + $starting_theta[$l] + $s*$dtheta[$l] ; }
 if($t == 2){ $theta     = 90 + $dth + $starting_theta[$l] + $s*$dtheta[$l] ; }

 my $x         = sprintf("%.3f",$R*cos(rad($theta)));
 my $y         = sprintf("%.3f",$R*sin(rad($theta)));
 return "$x*mm $y*mm";
}


for(my $l = 0; $l < $nlayers; $l++)
{
 my $layer_no       = $l + 1;

	place_chip($l, 1, 1);
	place_chip($l, 1, 2);
	place_wirebond($l, 1);
	place_pcboard($l, 1);
	place_pitch_adapter($l, 1);
	place_silicon($l, 1);
	place_epoxy($l, 1);
	place_bus_cable($l, 1);
	place_carbon_fiber($l, 1);
	place_rohacell($l);
	place_copper_support($l);
	place_carbon_fiber($l, 2);
	place_epoxy($l, 2);
	place_bus_cable($l, 2);
	place_silicon($l, 2);
	place_pitch_adapter($l, 2);
	place_pcboard($l, 2);
	place_wirebond($l, 2);
	place_chip($l, 2, 1);
	place_chip($l, 2, 2);
	place_downstream_support($l); 
}




# BST is a Tube. SLs are Tubes inside BST


make_bst();

for(my $l = 0; $l < $nlayers; $l++)
{
	make_sl($l+1);
	make_fluxes($l+1);
}

sub make_bst
{
 $detector{"name"}        = $envelope;
 $detector{"mother"}      = "root";
 $detector{"description"} = "Barrel Silicon Vertex Tracker";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "aaaaff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$bst_ir*mm $bst_or*mm $bst_dz*mm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}

sub make_sl
{
 my $slnumber = shift;
 my $slindex  = $slnumber - 1;

 $detector{"name"}        = "SL_$slnumber";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "Super Layer $slnumber";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "aaaaff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$SL_ir[$slindex]*mm $SL_or[$slindex]*mm $SL_dz*mm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}





sub make_fluxes
{
	my $slnumber = shift;
	my $slindex  = $slnumber - 1;

	my $RIN = $SL_or[$slindex] + 2;
	my $ROU = $RIN + 0.1;


	$detector{"name"}        = "flux_$slnumber";
	$detector{"mother"}      = $envelope;
	$detector{"description"} = "Super Layer $slnumber";
	$detector{"pos"}         = "0*cm 0*cm 0*cm";
	$detector{"rotation"}    = "0*deg 0*deg 0*deg";
	$detector{"color"}       = "aaaaff";
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "$RIN*mm $ROU*mm $SL_dz*mm 0*deg 360*deg";
	$detector{"material"}    = "Air";
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;		
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 0;
	$detector{"style"}       = 0;
	$detector{"sensitivity"} = "FLUX";
	$detector{"hit_type"}    = "FLUX";
	$detector{"identifiers"} = "if manual $slnumber";

	print_det(\%detector, $file);


	if($slnumber == 1)
	{
		my $RIN = $SL_ir[$slindex] -3;
		my $ROU = $RIN + 0.1;
		$detector{"name"}        = "flux_0";
		$detector{"mother"}      = $envelope;
		$detector{"description"} = "Super Layer $slnumber";
		$detector{"pos"}         = "0*cm 0*cm 0*cm";
		$detector{"rotation"}    = "0*deg 0*deg 0*deg";
		$detector{"color"}       = "aaaaff";
		$detector{"type"}        = "Tube";
		$detector{"dimensions"}  = "$RIN*mm $ROU*mm $SL_dz*mm 0*deg 360*deg";
		$detector{"material"}    = "Air";
		$detector{"mfield"}      = "no";
		$detector{"ncopy"}       = 1;		
		$detector{"pMany"}       = 1;
		$detector{"exist"}       = 1;
		$detector{"visible"}     = 0;
		$detector{"style"}       = 0;
		$detector{"sensitivity"} = "FLUX";
		$detector{"hit_type"}    = "FLUX";
		$detector{"identifiers"} = "if manual 0";

		print_det(\%detector, $file);

	}

}




sub place_silicon
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r = r_placement($l, 1);
    $vname      = "BST_SiV_Layer";
    $descriptio = "Barrel Silicon Tracker, Silicon V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r = r_placement($l, 9); 
    $vname      = "BST_SiW_Layer";
    $descriptio = "Barrel Silicon Tracker, Silicon W, Layer $layer_no, ";
 }

 for(my $m = 0; $m < $number_of_modules[$l]; $m++)
 {
    my $module_n = $m + 1;

    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
       my $snumber   = cnumber($s, 10);
       my $tpos      = pos_t($r, $l, $s);
       my $z         = $starting_point[$l] + $module_length / 2 + $m*$module_length + $m*$microgap_width + $m*$silicon_gap_width;
       my $Dx        = $module_height / 2;
       my $Dy        = $silicon_width / 2;
       my $Dz        = $module_length / 2;
       my $segment_n = $s + 1;

       $detector{"name"}        = "$vname$layer_no\_Module$module_n\_Segment$snumber";
       $detector{"mother"}      = "SL_$layer_no";
       $detector{"description"} = "$descriptio Module $module_n, Segment $segment_n";
       $detector{"pos"}         = "$tpos $z*mm";
       $detector{"rotation"}    = rot($r, $l, $s);
       $detector{"color"}       = $silicon_color[$m];
       $detector{"type"}        = "Box";
       $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
       $detector{"material"}    = $silicon_material;
       $detector{"mfield"}      = "no";
       $detector{"ncopy"}       = $s + 1;
       $detector{"pMany"}       = 1;
       $detector{"exist"}       = 1;
       $detector{"visible"}     = 1;
       $detector{"style"}       = 1;
       $detector{"sensitivity"} = "BST";
       $detector{"hit_type"}    = "BST";
       $detector{"identifiers"} = "superlayer manual $layer_no type manual $type segment manual $detector{'ncopy'} module manual $module_n strip manual 1";

       print_det(\%detector, $file);
    }
 }
}

# Bus Cable goes from the hip to the end of the silicon
sub place_bus_cable
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r = r_placement($l, 3);
    $vname      = "BST_BusCable_V_Layer";
    $descriptio = "BusCable V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r = r_placement($l, 7);
    $vname      = "BST_BusCable_W_Layer";
    $descriptio = "BusCable W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $module_height / 2;
    my $Dy        = $bus_cable_width / 2;
    #my $Dz        = $carbon_fiber_length / 2.0   ;
    my $Dz        = ( $number_of_modules[$l]*$module_length + $number_of_modules[$l]*$microgap_width + $pcboard_length + $microgap_width) / 2.0 + 2;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] + $roh_carbon_gap + $Dz - 46.58 - 4 - 0.11;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $bus_cable_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $bus_cable_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}


# Epoxy goes from the hip to the end of the silicon
sub place_epoxy
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r = r_placement($l, 2);
    $vname      = "BST_Epoxy_V_Layer";
    $descriptio = "Epoxy V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r = r_placement($l, 8);
    $vname      = "BST_Epoxy_W_Layer";
    $descriptio = "Epoxy W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $module_height / 2;
    my $Dy        = $epoxy_width / 2;
    my $Dz        = ( $number_of_modules[$l]*$module_length + $number_of_modules[$l]*$microgap_width + $pcboard_length + $microgap_width) / 2.0 + 2;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] + $roh_carbon_gap + $Dz - 46.58 - 4 - 0.11;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $epoxy_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $epoxy_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}

sub place_carbon_fiber
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r = r_placement($l, 4);
    $vname       = "BST_CarbonFiber_V_L";
    $descriptio  = "Epoxy V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r = r_placement($l, 6);
    $vname       = "BST_CarbonFiber_W_L";
    $descriptio  = "Epoxy W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $module_height / 2;
    my $Dy        = $carbon_fiber_width / 2;
    my $Dz        = $carbon_fiber_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] + $roh_carbon_gap + $Dz - 46.58 - 4 - 0.11;   # copper extension + pitch adapter + silicon gap
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $carbon_fiber_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $carbon_fiber_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}



sub place_pitch_adapter
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r           = r_placement($l, 1) ;
    $vname       = "BST_Pitch_adapter_V_L";
    $descriptio  = "PC Board V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r           = r_placement($l, 9)   ; 
    $vname       = "BST_Pitch_adapter_W_L";
    $descriptio  = "PC Board W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $module_height / 2.0;
    my $Dy        = $silicon_width  / 2.0;
    my $Dz        = $pitch_adapter_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] - $microgap_width - $Dz - $silicon_gap_width;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $paboard_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $paboard_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}



sub place_rohacell
{
	my $l    = shift;
	my $layer_no       = $l + 1;
	my $r = r_placement($l, 5);

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $rohacell_height / 2;
    my $Dy        = $rohacell_width / 2;
    my $Dz        = $rohacell_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] + $Dz + 2.0;   # 2mm is now copper   
    my $segment_n = $s + 1;

    $detector{"name"}        = "BST_Rohacell_L$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "Rohacell support, Layer $layer_no, Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $rohacell_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $rohacell_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
		

		
    $Dx  = $plastic_height / 2.0;
    $Dy  = $plastic_width / 2.0 ;
    $Dz  = $plastic_length / 2.0 ;
    $z = $rohacell_length / 2.0 - $plastic_length/2.0 - 0.1;
    $detector{"name"}        = "BST_Plastic_L$layer_no\_Segment$snumber";
    $detector{"mother"}      = "BST_Rohacell_L$layer_no\_Segment$snumber";
    $detector{"description"} = "Plastic Tab inside Rohahell, Layer $layer_no, Segment $segment_n";
    $detector{"pos"}         = "0*mm 0*mm $z*mm";
    $detector{"rotation"}    = "0";
    $detector{"color"}       = $epoxy_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $rohacell_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
		
		
		
		
		
		
		
 }
}

sub place_copper_support
{
	my $l    = shift;
	my $layer_no       = $l + 1;
	my $r = r_placement($l, 5);

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $bst_support_height / 2.0;
    my $Dy        = $bst_support_width  / 2.0;
    my $Dz        = $bst_copper_length[$l] / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] + $Dz - $microgap_width - $bst_copper_length[$l] - 2.11;
    my $segment_n = $s + 1;

    $detector{"name"}        = "BST_Copper_Support_L$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "Rohacell support, Layer $layer_no, Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $bst_support_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $bst_support_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}

sub place_wirebond
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r          = r_placement($l, 1)  - $wirebond_width / 2.0 - $microgap_width - $silicon_width;
    $vname      = "BST_Wirebond_V_Layer";
    $descriptio = "Barrel Silicon Tracker, Wirebond V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r          = r_placement($l, 9) + $wirebond_width / 2.0 + $microgap_width + $silicon_width;  
    $vname      = "BST_Wirebond_W_Layer";
    $descriptio = "Barrel Silicon Tracker, Wirebond W, Layer $layer_no, ";
 }

 for(my $m = 0; $m < $number_of_modules[$l]; $m++)
 {
    my $module_n = $m + 1;
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
       my $snumber   = cnumber($s, 10);
       my $Dx        = $pcboard_height  / 2.0;
       my $Dy        = $wirebond_width  / 2.0;
       my $Dz        = $wirebond_length / 2.0;
       my $tpos      = pos_t($r, $l, $s);
       my $z         = $starting_point[$l] + $m*$module_length - $Dz + $wirebond_length / 2.0 ;
       my $segment_n = $s + 1;

       $detector{"name"}        = "$vname$layer_no\_Module$module_n\_Segment$snumber";
       $detector{"mother"}      = "SL_$layer_no";
       $detector{"description"} = "$descriptio Module $module_n, Segment $segment_n";
       $detector{"type"}        = "Box";
       $detector{"pos"}         = "$tpos $z*mm";
       $detector{"rotation"}    = rot($r, $l, $s);
       $detector{"color"}       = $wirebond_color;
       $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
       $detector{"material"}    = $silicon_material;
       $detector{"mfield"}      = "no";
       $detector{"ncopy"}       = $s + 1;
       $detector{"pMany"}       = 1;
       $detector{"exist"}       = 1;
       $detector{"visible"}     = 1;
       $detector{"style"}       = 1;
       $detector{"sensitivity"} = "no";
       $detector{"hit_type"}    = "";
       $detector{"identifiers"} = "";

       print_det(\%detector, $file);
    }
 }
}

sub place_pcboard
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r           =  r_placement($l, 1) - $pcboard_width / 2.0 - $microgap_width ;
    $vname       = "BST_PCBoard_V_L";
    $descriptio  = "PC Board V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r           =  r_placement($l, 9)  + $pcboard_width / 2.0 + $microgap_width ; 
    $vname       = "BST_PCBoard_W_L";
    $descriptio  = "PC Board W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $pcboard_height / 2.0;
    my $Dy        = $pcboard_width  / 2.0;
    my $Dz        = $pcboard_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $starting_point[$l] - $microgap_width - $Dz - $pitch_adapter_length - 0.34;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $pcboard_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $pcboard_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}


sub place_chip
{
 my $l          = shift;
 my $type       = shift;
 my $left_rigth = shift;
 my $layer_no   = $l + 1;
 my $r1         = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r1           = r_placement($l, 1) - $pcboard_width - $chip_width / 2.0 - 2.0*$microgap_width ;
    $vname       = "BST_chip_$left_rigth\_V_L";
    $descriptio  = "chip V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r1          = r_placement($l, 9)  + $pcboard_width + $chip_width / 2.0 + 2.0*$microgap_width ; 
    $vname       = "BST_chip_$left_rigth\_W_L";
    $descriptio  = "chip W, Layer $layer_no, ";
 }
 my $r           = sqrt($r1 * $r1 + $chip_displacement * $chip_displacement);

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $chip_height / 2.0;
    my $Dy        = $chip_width  / 2.0;
    my $Dz        = $chip_length / 2.0 ;
    my $tpos      = pos_t_chip($r, $l, $left_rigth, $s);
    my $z         = $starting_point[$l] - $microgap_width - $Dz - $wirebond_length - - $microgap_width - $pitch_adapter_length;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $chip_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $pcboard_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "FLUX";
    $detector{"hit_type"}    = "FLUX";
    $detector{"identifiers"} = "id manual 22";

    print_det(\%detector, $file);
 }
}


sub place_downstream_support
{
	my $l          = shift;

	my @iradius  = (   0.0  ,     0.0   ,   0.0  ,   0.0);   # Inner radii of solenoid
	my @oradius  = (  65.39 ,    93.07  , 120.0 , 160.98);  # Outer radii of solenoid - ATTENTION: LAYER3 is 0.49mm less because of overlaps
	my $ring_dz = 14;
	my $nzplanes = 2;




	my @IR     = ($iradius[$l], $iradius[$l] ); 
	my @OR     = ($oradius[$l], $oradius[$l] ); 
	my @zplane = (0     , $ring_dz    ); 
	my $SL = $l + 1;
	my $z         = $starting_point[$l] + $rohacell_length  - $ring_dz  + 2.0;   # same as rohacell position
		
	
	my $dimensions = "0.0*deg 360.0*deg $nsegments[$l] $nzplanes";
	for(my $i = 0; $i <$nzplanes ; $i++)
	{
		$dimensions = $dimensions ." $IR[$i]*mm";
	}
	for(my $i = 0; $i <$nzplanes ; $i++)
	{
		$dimensions = $dimensions ." $OR[$i]*mm";
	}
	for(my $i = 0; $i <$nzplanes ; $i++)
	{
		$dimensions = $dimensions ." $zplane[$i]*mm";
	}


	$detector{"name"}        = "aBST_Downstream_SupportFrame_L$SL";
	$detector{"mother"}      = "root";
	$detector{"description"} = "Downstream support Frame, Layer $SL";
	$detector{"pos"}         = "0*mm 0*mm 0*mm";
	$detector{"rotation"}    = "0";
	$detector{"color"}       = $bst_support_color;
	$detector{"type"}        = "Pgon";
	$detector{"dimensions"}  = "$dimensions";
	$detector{"material"}    = "Component";
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";

	print_det(\%detector, $file);
	
	
	
	my @toradius  = (  62.39 ,    89.47  , 116.35 , 156.37);  # Outer radii of solenoid
	
	
	$detector{"name"}        = "bBST_Downstream_SupportHole_L$SL";
	$detector{"mother"}      = "root";
	$detector{"description"} = "Downstream support hole, Layer $SL";
	$detector{"pos"}         = "0*mm 0*mm 7*mm";
	$detector{"rotation"}    = "0";
	$detector{"color"}       = $bst_support_color;
	$detector{"type"}        = "Tube";
	$detector{"dimensions"}  = "0 $toradius[$l] $ring_dz 0*deg 360*deg";
	$detector{"material"}    = "Component";
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";

	print_det(\%detector, $file);
	
 
 	$detector{"name"}        = "cBST_Downstream_Support_L$SL";
	$detector{"mother"}      = "SL_$SL";
	$detector{"description"} = "Downstream support, Layer $SL";
	$detector{"pos"}         = "0*mm 0*mm $z*mm";
	$detector{"rotation"}    = rot(0, $l, 0);
	if($l == 3)	{$detector{"rotation"}    = rot(0, $l, 199);}

	$detector{"color"}       = $bst_support_color;
	$detector{"type"}        = "Operation: aBST_Downstream_SupportFrame_L$SL - bBST_Downstream_SupportHole_L$SL";
	$detector{"dimensions"}  = "0";
	$detector{"material"}    = $bst_support_material;
	$detector{"mfield"}      = "no";
	$detector{"ncopy"}       = 1;
	$detector{"pMany"}       = 1;
	$detector{"exist"}       = 1;
	$detector{"visible"}     = 1;
	$detector{"style"}       = 1;
	$detector{"sensitivity"} = "no";
	$detector{"hit_type"}    = "";
	$detector{"identifiers"} = "";

	print_det(\%detector, $file);
	

 
 
 
}








