// %%%%%%%%%%
// G4 headers
// %%%%%%%%%%
#include "G4UnitsTable.hh"
#include "G4Poisson.hh"
#include "Randomize.hh"

// %%%%%%%%%%%%%
// gemc headers
// %%%%%%%%%%%%%
#include "PCAL_hitprocess.h"

PH_output PCAL_HitProcess :: ProcessHit(MHit* aHit, gemc_opts)
{
	PH_output out;
	out.identity = aHit->GetId();
	
	
	// get sector, stack (inner or outer), view (U, V, W), and strip.
	int sector = out.identity[0].id;
	int layer  = out.identity[1].id;
	int view   = out.identity[2].id;
	int strip  = out.identity[3].id;
	HCname = "PCAL Hit Process";	
	
	
	// %%%%%%%%%%%%%%%%%%%
	// Raw hit information
	// %%%%%%%%%%%%%%%%%%%
	int nsteps = aHit->GetPos().size();
	
	// Get Total Energy deposited
	double Etot = 0;
	vector<G4double> Edep = aHit->GetEdep();
	for(int s=0; s<nsteps; s++) Etot = Etot + Edep[s];
	
	// average global, local positions of the hit
	double x, y, z;
	double lx, ly, lz;
	x = y = z = lx = ly = lz = 0;
	vector<G4ThreeVector> pos  = aHit->GetPos();
	vector<G4ThreeVector> Lpos = aHit->GetLPos();
	
	if(Etot>0)
		for(int s=0; s<nsteps; s++)
		{
			x  = x  +  pos[s].x()*Edep[s]/Etot;
			y  = y  +  pos[s].y()*Edep[s]/Etot;
			z  = z  +  pos[s].z()*Edep[s]/Etot;
			lx = lx + Lpos[s].x()*Edep[s]/Etot;
			ly = ly + Lpos[s].y()*Edep[s]/Etot;
			lz = lz + Lpos[s].z()*Edep[s]/Etot;
		}
		else
		{
			x  = pos[0].x();
			y  = pos[0].y();
			z  = pos[0].z();
			lx = Lpos[0].x();
			ly = Lpos[0].y();
			lz = Lpos[0].z();
		}
		
		// average time
		double time = 0;
		vector<G4double> times = aHit->GetTime();
		for(int s=0; s<nsteps; s++) time = time + times[s]/nsteps;
		
		// Energy of the track
		double Ene = aHit->GetE();
		
		out.raws.push_back(Etot);
		out.raws.push_back(x);
		out.raws.push_back(y);
		out.raws.push_back(z);
		out.raws.push_back(lx);
		out.raws.push_back(ly);
		out.raws.push_back(lz);
		out.raws.push_back(time);
		out.raws.push_back((double) aHit->GetPID());
		out.raws.push_back(aHit->GetVert().getX());
		out.raws.push_back(aHit->GetVert().getY());
		out.raws.push_back(aHit->GetVert().getZ());
		out.raws.push_back(Ene);
		out.raws.push_back((double) aHit->GetmPID());
		out.raws.push_back(aHit->GetmVert().getX());
		out.raws.push_back(aHit->GetmVert().getY());
		out.raws.push_back(aHit->GetmVert().getZ());
		
		
		// %%%%%%%%%%%%
		// Digitization
		// %%%%%%%%%%%%
		// adapted by Alex Piaseczny, Canisius College Medium Energy Nuclear Physics (CMENP), through
		// Dr. Michael Wood from Gilfoyle EC code
		// Jerry Gilfoyle, Feb, 2010
		
		// parameters: factor  - conversion for adc (MeV/channel).
		//             pcal_tdc_to_channel - conversion factor for tdc (ns/channel).
		int pcal_tdc_time_to_channel = 20;     // conversion from time (ns) to TDC channels.
		double PCALfactor = 35;                // number of photons divided by the energy deposited in MeV; value taken from gsim. see EC NIM paper table 1.
		int PCAL_TDC_MAX = 4095;               // max value for PCAL tdc.
		int pcal_charge_to_channel = 20;       // conversion from charge (pC) to ADC channels; taken from IC code.
		double pcal_npe_to_charge = 0.0005;    // unjustified (!!) conversion from number of photoelectrons to charge in pC.
		
		// initialize ADC and TDC
		int PCAL_ADC = 0;
		int PCAL_TDC = PCAL_TDC_MAX;
		
		// simulate the adc value.
		if (Etot > 0) {
			// number of photoelectrons.
			double PCAL_npe = G4Poisson(Etot*PCALfactor);
			//  Fluctuations in PMT gain distributed using Gaussian with sigma SNR = sqrt(ngamma)/sqrt(del/del-1) del = dynode gain = 3 (From RCA PMT Handbook) p. 169)
			//  algorithm, values, and comment above taken from gsim.
			double sigma = sqrt(PCAL_npe)*1.15;
			double PCAL_charge = G4RandGauss::shoot(PCAL_npe,sigma)*pcal_npe_to_charge*pcal_charge_to_channel;
			if (PCAL_charge <= 0) PCAL_charge=0.0; // guard against weird, rare events.
				PCAL_ADC = (int) PCAL_charge;
		}
		
		// simulate the tdc.
		PCAL_TDC = (int) (time*pcal_tdc_time_to_channel);
		if (PCAL_TDC > PCAL_TDC_MAX) PCAL_TDC = PCAL_TDC_MAX;
		
		out.dgtz.push_back(sector);
		out.dgtz.push_back(layer);
		out.dgtz.push_back(view);
		out.dgtz.push_back(strip);
		out.dgtz.push_back(PCAL_ADC);
		out.dgtz.push_back(PCAL_TDC);
		
// 		cout << "sector = " << sector << " layer = " << layer << " view = " << view << " strip = " << strip << " PCAL_ADC = " << PCAL_ADC << " PCAL_TDC = " << PCAL_TDC << " Edep = " << Etot << endl;
		
		return out;
		
}


vector<identifier>  PCAL_HitProcess :: ProcessID(vector<identifier> id, G4Step* aStep, detector Detector, gemc_opts Opt)
{
	return id;
}
// have to change end










