// %%%%%%%%%%
// G4 headers
// %%%%%%%%%%
#include "G4Box.hh"
#include "G4GeometryManager.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4LogicalVolume.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4ProductionCuts.hh"
#include "G4PVPlacement.hh"
#include "G4RunManager.hh"
#include "G4SolidStore.hh"
#include "G4VisAttributes.hh"
#include "G4SDManager.hh"
#include "G4OpBoundaryProcess.hh"

// %%%%%%%%%%%%
// gemc headers
// %%%%%%%%%%%%
#include "MDetectorConstruction.h"
#include "MDetectorMessenger.h"

// %%%%%%%%%%%
// C++ headers
// %%%%%%%%%%%
#include <sstream>
using namespace std;

MDetectorConstruction::MDetectorConstruction(gemc_opts Opts)
{
	gemcOpt = Opts;
	// mdetectorMessenger = new MDetectorMessenger(this);
}

MDetectorConstruction::~MDetectorConstruction()
{
	;
}


G4VPhysicalVolume* MDetectorConstruction::Construct()
{
	string hd_msg     = gemcOpt.args["LOG_MSG"].args + " Construction: >> ";
	double VERB       = gemcOpt.args["G4P_VERBOSITY"].arg ;
	string catch_v    = gemcOpt.args["CATCH"].args;
	string hall_mat   = gemcOpt.args["HALL_MATERIAL"].args;
	string hall_field = gemcOpt.args["HALL_FIELD"].args;


	// Clean old geometry, if any
	G4GeometryManager::GetInstance()->OpenGeometry();
	G4PhysicalVolumeStore::GetInstance()->Clean();
	G4LogicalVolumeStore::GetInstance()->Clean();
	G4SolidStore::GetInstance()->Clean();


	// Experimental hall is a 40 meters box
	detector queen;
	queen.name        = "root";
	queen.mother      = "akasha";
	queen.description = "mother of us all";
	queen.pos         =  G4ThreeVector(0, 0, 0);
	queen.rot         =  G4RotationMatrix(G4ThreeVector(1, 0, 0),
                                        G4ThreeVector(0, 1, 0),
                                        G4ThreeVector(0, 0, 1));
	queen.type        =  "Box";
	queen.dimensions.push_back(40*m);
	queen.dimensions.push_back(40*m);
	queen.dimensions.push_back(40*m);
	queen.material    = hall_mat;
	queen.magfield    = hall_field;
	queen.exist       = 1;
	queen.visible     = 0;
	queen.ncopy       = 0;
	queen.scanned     = 1;
	queen.create_solid(gemcOpt, Hall_Map);
	queen.create_logical_volume(MMats, gemcOpt);
	queen.create_physical_volumes(gemcOpt, NULL);
	HasMagfield(queen,  gemcOpt);

	(*Hall_Map)["root"] = queen;

	if(VERB > 3 || catch_v == "root") cout << hd_msg << "    " << queen ;


	cout << hd_msg <<  " Building Detector from Geometry STL Map..." << endl << endl;

	// ######################################################
	// Resetting Detector Map "scanned". Propagating "exist".
	// ######################################################
	if(VERB > 2) cout << hd_msg << " Mapping Physical Detector..." << endl << endl;

	for( map<string, detector>::iterator i =  Hall_Map->begin(); i!=Hall_Map->end(); i++)
	{
		if(VERB > 3) cout << hd_msg << " Scanning Detector " << i->first << " - existance: " << i->second.exist << endl;

		// Propagating "exist" flag to child - just in case
		if(i->second.mother != "akasha")
		{
			map<string,detector>::iterator it = Hall_Map->find(i->second.mother); // Find the mother

			if(it != Hall_Map->end() && it->second.exist == 0 && i->second.exist == 1)
			{
				if(VERB > 2) cout << hd_msg <<   "\t" << i->second.mother  << " is not activated. Its child " << i->second.name
					<< " will be disactivated as well." << endl;
				i->second.exist = 0;
			}
			if(i->first != "root") i->second.scanned = 0;
		}
	}
	if(VERB > 2) cout << endl;

	// ########################################################################
	// Building Solids, Logical Volumes, Physical Volumes from the detector Map
	// ########################################################################
	vector<string> relatives;
	string mom, kid;
	for( map<string, detector>::iterator i =  Hall_Map->begin(); i!=Hall_Map->end(); i++)
	{
		// don't build anything if it doesn't exist
		if(i->second.exist == 0) continue;
		
		// put the first volume in relatives
		// typically its the first in alphabetical order
		if(i->first != "root") relatives.push_back(i->second.name);
		
		int max_hierarchy = 0;
		while(relatives.size() > 0)
		{
			mom = (*Hall_Map)[relatives.back()].mother;  // mother of last volume in relativea
			kid = (*Hall_Map)[relatives.back()].name;    // last volume name
			
			// hardcoding a max hierarchy of 40
			if(++max_hierarchy>40)
			{
				cout << hd_msg << " Reached limits of 40 max hierarchy for <" << kid << ">. Exiting." << endl;
				exit(0);
			}
			
       // Mom doesn't exists in the Hall_Map. Stopping everything.
			if((*Hall_Map)[mom].name == "" && mom != "akasha"  && (*Hall_Map)[mom].scanned == 0)
			{
				cout << hd_msg << "  Mom <" << mom << "> was not created for <" << kid << ">. "
											 << ">. We have a No Child Left Behind policy. Exiting. " << endl << endl;
				exit(0);
			}

			// output the Geneaology
			if(VERB > 3 || kid.find(catch_v) != string::npos)
			{
				for(unsigned int ir=0; ir<relatives.size()-1; ir++) cout << "\t";
				cout << hd_msg << " Checking " << kid << ", child of " << mom
											 << ", for a living ancestor. "
											 << " This Geneaology Depth is " << relatives.size() << "." << endl;
			}

			// Mom is built, kid not built yet. Build the kid.
			if((*Hall_Map)[kid].scanned == 0 && (*Hall_Map)[mom].scanned == 1)
			{
				if(VERB > 3 || kid.find(catch_v) != string::npos)
				{
					for(unsigned int ir=0; ir<relatives.size()-1; ir++) cout << "\t";
					cout << hd_msg << "  Found:  " << kid
					               << " is not built yet but its mommie " << mom << " is." 
												 << " Building " << kid << "..."  << endl;
				}
				// creating solid
				(*Hall_Map)[kid].create_solid(gemcOpt, Hall_Map);
				// creating logical volume
				if((*Hall_Map)[kid].create_logical_volume(MMats, gemcOpt))
				{
					// creating physical volume
					(*Hall_Map)[kid].create_physical_volumes(gemcOpt, (*Hall_Map)[mom].GetLogical());
					IsSensitive((*Hall_Map)[kid], gemcOpt);
					HasMagfield((*Hall_Map)[kid], gemcOpt);
				}
				(*Hall_Map)[kid].scanned = 1;
			}

			// if the kid still doesn't exists it means its mom doesn't exist. Need to go up one level.
			if((*Hall_Map)[kid].scanned == 0 && mom != "akasha") 
			{
				relatives.push_back(mom);
				string type((*Hall_Map)[mom].type);
				// if the mother is an operation, we need to build the operand first
				if(type.find("Operation:") == 0)
				{
					stringstream ops;
					string operands(type, 10, type.size());
					ops << operands;
					string firstop, secondop, tmpop;
					ops >>  firstop >> tmpop >> secondop;
					relatives.push_back(secondop);
					relatives.push_back(firstop);
										
					if(VERB > 3)
					{
						for(unsigned int ir=0; ir<relatives.size()-1; ir++) cout << "\t";
						cout << hd_msg << mom << " is an composite volume. "
						                      << " Must build: " << firstop << " and " << secondop << endl;					
					}					
				}
			}

			// the kid has been built. Can go down one step in geneaology
			if((*Hall_Map)[kid].scanned == 1 && relatives.size())
			{
				if(VERB > 3 || kid.find(catch_v) != string::npos)
					cout << hd_msg  << " " <<  kid << " is built." <<  endl << endl;

				relatives.pop_back();
			}
		}
	}

	// %%%%%%%
	// Mirrors
	// %%%%%%%
	
	// creating STL vector of Mirrors. 
  // the array of photon energies has size "2"
  // but all mirror properties are independent of energy
	const int nEntries_Mirror = 2;
	double PhotonEnergy_Mirror[nEntries_Mirror] = { 1.034*eV, 5.144*eV };
	
  vector<G4OpticalSurface*>          Mirror_Surfaces;
	vector<G4MaterialPropertiesTable*> Mirror_MPT;
	vector<G4LogicalBorderSurface*>    Mirrors;
	vector<G4LogicalSkinSurface*>      SMirrors;



	// now going through all volumes with the Identifier set to "Mirror:"
	// the second entry is the volume that borders with the one considered
	// the third entry is the reflectivity (double number, between zero and one)
	// the fourth entry is the refractive index (double number, between zero and one)
	for( map<string, detector>::iterator i =  Hall_Map->begin(); i!=Hall_Map->end(); i++)
	{	
		// looking for Mirrors
    if(i->second.identity.size())
      if(i->second.identity[0].name == "Mirror" && i->second.identity[0].rule == "WithSurface:")
      {
        string name        = i->first;
        string borderv     = "";
        string model[2]    = {"unified", "LUT"};
        string finish[2]   = {"polished", "ground"};
        string surfaces[2] = {"dielectric_metal", "dielectric_dielectric"};
        double refraction  = 1.0;
        double efficiency  = 1.0;
        double reflection  = 1.0;
        int surfaceType    = 0;
        int finishType     = 0;
        int modelType      = 0;

        for(unsigned int j=0; j<i->second.identity.size(); j++)
        {
                    
          // Border Volume
          if(i->second.identity[j].name == "WithBorderVolume:")
          {
            borderv = i->second.identity[j].rule;
          }
          
          // Surface Type:
          // 0: dielectric dielectric
          // 1: metal dielectric
          if(i->second.identity[j].rule == "WithSurface:")
          {
            surfaceType = i->second.identity[j].id;
          }
         
          // Finish
          // 0: polished
          // 1: ground
          if(i->second.identity[j].name == "With" && i->second.identity[j].rule == "Finish:")
          {
            finishType = i->second.identity[j].id;
          }
          
          
          // Refraction Index is ID / 1,000,000
          if(i->second.identity[j].name == "Refraction" && i->second.identity[j].rule == "Index:")
          {
            refraction = i->second.identity[j].id/1000000.0;
          }

          // Reflectivity is ID / 1,000,000
          if(i->second.identity[j].name == "With" && i->second.identity[j].rule == "Reflectivity:")
          {
            reflection = i->second.identity[j].id/1000000.0;
          }

          // Efficiency is ID / 1,000,000
          if(i->second.identity[j].name == "With" && i->second.identity[j].rule == "Efficiency:")
          {
            efficiency = i->second.identity[j].id/1000000.0;
          }
          
          // Model 
          // 0: unified
          // 1: LUT
          if(i->second.identity[j].name == "With" && i->second.identity[j].rule == "Model:")
          {
            modelType = i->second.identity[j].id;
          }
         
          
        }
        

        if(!(*Hall_Map)[borderv].GetPhysical() && borderv != "MirrorSkin")
        {
          cout << hd_msg << " Border volume " << borderv << " is not found for volume " << name << ". Exiting." << endl;
          exit(0);
        }
        
        string mirror_surf = "Mirror_Surface_for_" + name + "_and_" + borderv;
        if(borderv == "MirrorSkin")
        {
        	mirror_surf = "Mirror_Skin_Surface_for_" + name;
        }
        
        
        Mirror_Surfaces.push_back(new G4OpticalSurface(mirror_surf));
        Mirror_MPT.push_back(new G4MaterialPropertiesTable());
        
        // surface type
        if(surfaceType == 0) Mirror_Surfaces.back()->SetType(dielectric_metal);
        if(surfaceType == 1) Mirror_Surfaces.back()->SetType(dielectric_dielectric);
        
        // surface finish
        if(finishType == 0) Mirror_Surfaces.back()->SetFinish(polished);
        if(finishType == 1) Mirror_Surfaces.back()->SetFinish(ground);
        
        // surface model
        if(modelType == 0) Mirror_Surfaces.back()->SetModel(unified);
        if(modelType == 1) Mirror_Surfaces.back()->SetModel(LUT);
      
        
        double r_index[2] = {refraction, refraction};
        double reflect[2] = {reflection, reflection};
        double efficie[2] = {efficiency, efficiency};
        double spike[2] = {1.0, 1.0};
        double slobe[2] = {0.0, 0.0};
        double sback[2] = {0.0, 0.0};
        double sdiff[2] = {0.0, 0.0};
       
        Mirror_MPT.back()->AddProperty("RINDEX",       PhotonEnergy_Mirror, r_index, nEntries_Mirror);
        Mirror_MPT.back()->AddProperty("REFLECTIVITY", PhotonEnergy_Mirror, reflect, nEntries_Mirror);
        Mirror_MPT.back()->AddProperty("EFFICIENCY",   PhotonEnergy_Mirror, efficie, nEntries_Mirror);
        Mirror_MPT.back()->AddProperty("SPECULARSPIKECONSTANT", PhotonEnergy_Mirror, spike, nEntries_Mirror);
        Mirror_MPT.back()->AddProperty("SPECULARLOBECONSTANT",  PhotonEnergy_Mirror, slobe, nEntries_Mirror);
        Mirror_MPT.back()->AddProperty("BACKSCATTERCONSTANT",   PhotonEnergy_Mirror, sback, nEntries_Mirror);
        Mirror_MPT.back()->AddProperty("DIFFUSELOBECONSTANT",   PhotonEnergy_Mirror, sdiff, nEntries_Mirror);
        
        Mirror_Surfaces.back()->SetMaterialPropertiesTable(Mirror_MPT.back());

        if(borderv=="MirrorSkin")
        {
          SMirrors.push_back(new G4LogicalSkinSurface(name,
                                                      (*Hall_Map)[name].GetLogical(), Mirror_Surfaces.back()));
        }
				else
        {
        
        	Mirrors.push_back(new G4LogicalBorderSurface(name,
                                (*Hall_Map)[borderv].GetPhysical(),
                                (*Hall_Map)[name].GetPhysical(), Mirror_Surfaces.back()));
        }

        
        if(VERB > 3 || name.find(catch_v) != string::npos)
        {
          cout << hd_msg  << " " <<  name << " is a mirror:" << endl;
          cout << "                             > Border Volume: "      << borderv << endl;
          cout << "                             > Surface Type: "       << surfaces[surfaceType] << endl;
          cout << "                             > Finish: "             << finish[finishType] << endl;
          cout << "                             > Refraction Index: "   << refraction << endl;
          cout << "                             > Reflectivity: "       << reflection << endl;
          cout << "                             > Efficiency: "         << efficiency << endl;
          cout << "                             > Model: "              << model[modelType] << endl;
          
          
          Mirror_MPT.back()->DumpTable();

  
        }
      }  
	}	

	return (*Hall_Map)["root"].GetPhysical();
}

#include "G4UserLimits.hh"

void MDetectorConstruction::IsSensitive(detector detect, gemc_opts Opts)
{
	string hd_msg  = gemcOpt.args["LOG_MSG"].args + " Sensitivity: >> ";
	double VERB    = gemcOpt.args["HIT_VERBOSITY"].arg ;
	string catch_v = gemcOpt.args["CATCH"].args;

	string sensi   = detect.sensitivity;

	if(sensi != "no")
	{
    // Sensitive_region->AddRootLogicalVolume(detect.GetLogical());
		G4SDManager* SDman = G4SDManager::GetSDMpointer();
		if (!SDman) throw "Can't get G4SDManager";

		if(VERB > 5 || detect.name.find(catch_v) != string::npos)
			cout << hd_msg  << " " <<  detect.name << " is sensitive."  << endl;

    // The key for the SD, Region, PC maps is the same so we can only check SD
		map<string, MSensitiveDetector*>::iterator itr = SeDe_Map.find(sensi);
		if(itr == SeDe_Map.end())
		{
			if(VERB > 3 || detect.name.find(catch_v) != string::npos)
				cout << endl << hd_msg  << " Sensitive detector <" << sensi
						<< "> doesn't exist yet. Adding <" << sensi << ">. " << endl;

			SeDe_Map[sensi] = new MSensitiveDetector(sensi, gemcOpt);

      // Creating G4 Region, assigning Production Cut to it.
			SeRe_Map[sensi] = new G4Region(sensi);
			SePC_Map[sensi] = new G4ProductionCuts;
			SePC_Map[sensi] ->SetProductionCut(SeDe_Map[sensi]->SDID.ProdThreshold);
			SeRe_Map[sensi]->SetProductionCuts(SePC_Map[sensi]);

			if(VERB > 3 || detect.name.find(catch_v) != string::npos)
				cout << hd_msg  << " Max Step for Sensitive detector <" << sensi
             << ">: " << SeDe_Map[sensi]->SDID.MaxStep/mm << " mm." <<  endl
             << hd_msg  << " Production Cut for Sensitive detector <" << sensi
             << ">: " << SeDe_Map[sensi]->SDID.ProdThreshold/mm << " mm." << endl << endl;

      // Pass Detector Map Pointer to Sensitive Detector
			SeDe_Map[sensi]->Hall_Map        = Hall_Map;

			SDman->AddNewDetector( SeDe_Map[sensi]);

		}
		detect.setSensitivity(SeDe_Map[sensi]);

    // Setting Max Acceptable Step for this SD
		detect.SetUserLimits(new G4UserLimits(SeDe_Map[sensi]->SDID.MaxStep, SeDe_Map[sensi]->SDID.MaxStep));

		map<string, G4Region*>::iterator  itrRE = SeRe_Map.find(sensi);
			if(itrRE != SeRe_Map.end()) SeRe_Map[sensi]->AddRootLogicalVolume(detect.GetLogical());
		else
		{
			cout << hd_msg << " Attention: " << sensi << " doesn't exist in the sensitive detector list. Aborting. " << endl;
			exit(9);
		}
	
	}

}


void MDetectorConstruction::HasMagfield(detector detect, gemc_opts Opts)
{
	string hd_msg  = gemcOpt.args["LOG_MSG"].args + " Magnetic Field: >> ";
	double VERB    = gemcOpt.args["MGN_VERBOSITY"].arg ;
	string catch_v = gemcOpt.args["CATCH"].args;
	string field   = gemcOpt.args["NO_FIELD"].args;

	if(field == "all" || detect.name.find(field) != string::npos)
		return;
	
	string magf   = detect.magfield;

	if(magf != "no")
	{
		if(VERB > 5 || detect.name.find(catch_v) != string::npos)
			cout << hd_msg  << " " <<  detect.name << " is inside "  << magf << " magnetic field." << endl;

		map<string, MagneticField>::iterator itr = FieldMap->find(magf);
		if(itr == FieldMap->end())
		{
			cout << hd_msg << " Magnetic Field <" << magf
					<< "> is not defined. Exiting." << endl;
			exit(0);
		}
		if(itr->second.get_MFM() == NULL)
		{
			cout << hd_msg << " Magnetic Field <" << magf
					<< "> doesn't exist yet. Adding <" << magf << ">. " << endl;
			itr->second.create_MFM();
		}

		if(itr->second.get_MFM() != NULL)
		{
			detect.AssignMFM(itr->second.get_MFM());

			if(VERB > 6 || detect.name.find(catch_v) != string::npos)
				cout << hd_msg  << " Field <" <<  magf << "> assigned to " << detect.name << "." << endl;

		}
	}

}


void MDetectorConstruction::UpdateGeometry()
{
	cout << "Updating geometry... " << endl;
	G4RunManager::GetRunManager()->GeometryHasBeenModified();
}





